import java.util.*;

        // Allow varying numbers of Students to be sorted a
        // varying number of times. Print the mean time to sort
        // a single array of Students.
class SortingMain {
    public static void main(String[] args) throws NumberFormatException {
        // The number of students and the number of sorts may
        // be altered by the command line arguments.
        int howManySorts = 100;
        int numStudents = 10000;

        switch(args.length){
            case 0: // Use the default values.
                break;
            case 2:
                numStudents = Integer.parseInt(args[1]);
                if(numStudents <= 0){
                     System.err.println(
                        "Number of students must be positive.");
                     System.exit(1);
                }
                // FALLTHROUGH.
            case 1:
                howManySorts = Integer.parseInt(args[0]);
                if(howManySorts <= 0){
                     System.err.println(
                        "Number of sorts must be positive.");
                     System.exit(1);
                }
                break;
            default:
                System.err.println(
                    "Usage: java SortingMain [#sorts [#students]]");
                System.exit(0);
        }

        // Generate random student data.
        final Random rand = new Random();
        final Student[] randomStudents = new Student[numStudents];
        for(int i = 0; i < numStudents; i++){
            randomStudents[i] = randomStudent(rand);
        }
        // Find the mean time for a single sort.
        double meanTime = sortTheData(randomStudents,howManySorts);
        System.out.println("The mean time for "+numStudents+
                           " items is "+meanTime+" milliseconds.");
    }

        // Generate a random student.
    public static Student randomStudent(Random rand){
        final int lettersInAlphabet = 26;
        final int maxMark = 100;
        final int nameLength = 10;
        final int mark = rand.nextInt(maxMark+1);
        // Generate a random name, with an initial upper
        // case letter.
        StringBuffer name = new StringBuffer(nameLength);
        name.append((char)('A'+rand.nextInt(lettersInAlphabet)));
        for(int i = 1; i < nameLength; i++){
            name.append((char)('a'+rand.nextInt(lettersInAlphabet)));
        }
        return new Student(name.toString(),mark);
    }

        // Return the mean time it takes for a single sort
        // of the given data.
    public static double sortTheData(Comparable[] randomData,
                                         int howManySorts){
        final StopWatch timer = new TimingDevice();
        final int numItems = randomData.length;
        // Take a copy of the randomized data for each
        // sort.
        Comparable[] dataToSort = new Comparable[numItems];
        System.arraycopy(randomData,0,dataToSort,0,numItems);
        timer.start();
        timer.suspend();
        for(int i = 0; i < howManySorts; i++){
            timer.resume();
            Arrays.sort(dataToSort);
            timer.suspend();
            System.arraycopy(randomData,0,dataToSort,0,numItems);
        }
        timer.stop();
        return timer.getElapsedTime()/(double)howManySorts;
    }
}
