    // Maintain a prioritized queue of objects.
public class PriorityQueue {
        // Add an object to the queue so that items with the highest
        // priority are held nearest the queue's head.
    public void add(Object o,int priority){
        Node n = new Node(o,priority);
        Node head = getHead();
        if(head == null){
            // Whole queue is empty.
            setHead(n);
        }
        else if(head.getPriority() < priority){
            // Insertion at the head.
            n.setNext(head);
            setHead(n);
        }
        else{
            // Work along the queue until the next in line has
            // a lower priority than the one to be inserted.
            // Keep track of the previous node, which will
            // hold the final insertion point.
            Node previous = head, next = previous.getNext();
            while(next != null && next.getPriority() >= priority){
                previous = next;
                next = previous.getNext();
            }
            // Insert after previous.
            previous.setNext(n);
            n.setNext(next);
        }
        setLength(getLength()+1);
    }

        // Return (and remove) the object at the head of the queue.
        // Return null if the queue is empty.
    public Object next(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
            // Drop the node from the head.
            Node nextNode = head.getNext();
            setHead(nextNode);
            setLength(getLength()-1);
        }
        return item;
    }

        // Return a reference to the object at the head of the queue,
        // without removing it.
        // Return null if the queue is empty.
    public Object peek(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
        }
        return item;
    }

        // Return the length of the queue.
    public int size(){
        return getLength();
    }
        
        // Maintain a reference to a single item in the queue, and
        // a reference to the next in line.
        // Each item has an associated (fixed) priority.
    protected class Node {
        public Node(Object item,int priority){
            this.item = item;
            this.priority = priority;
        }

        public Object getItem(){
            return item;
        }

        public int getPriority(){
            return priority;
        }

        public Node getNext(){
            return next;
        }

        public void setNext(Node n){
            next = n;
        }

        private final Object item;
        private final int priority;
        private Node next = null;
    }

    protected Node getHead(){
        return head;
    }

    protected void setHead(Node n){
        head = n;
    }

    protected int getLength(){
        return length;
    }

    protected void setLength(int len){
        length = len;
    }

    // Reference to the node at the head of the queue.
    private Node head = null;
    // The current length of the queue.
    private int length = 0;
}
